﻿REM //////////////////////////////////////////////////////////////////////////////////

REM Add the import for the InteropServices.  This is necessary for working with Arrays
Imports System.Runtime.InteropServices

Public Class SDKTester
    REM *******************************************************************************************************
    REM Declare the object as the SA SDK Interface
    REM *******************************************************************************************************
    Private NRKSdk As SpatialAnalyzerSDK.SpatialAnalyzerSDK

    REM *******************************************************************************************************
    REM Declare an enumerated type that defines the various return codes from the SA SDK Interface
    REM *******************************************************************************************************
    Enum MPStatus
        SdkError = -1
        Undone = 0
        InProgress = 1
        DoneSuccess = 2
        DoneFatalError = 3
        DoneMinorError = 4
        CurrentTask = 5
    End Enum

    REM *******************************************************************************************************
    REM Allocate the SDK Interface object
    REM *******************************************************************************************************
    Private Sub Form_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        NRKSdk = New SpatialAnalyzerSDK.SpatialAnalyzerSDK
    End Sub


    REM *******************************************************************************************************
    REM Connect to the SA SDK Server.  This must be done prior to attempting to run any MP Step
    REM *******************************************************************************************************
    Private Sub ConnectButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ConnectButton.Click
        If (NRKSdk.Connect("localhost") = False) Then
            MsgBox("Failed to connect with automation server!")
        End If
    End Sub

    REM *******************************************************************************************************
    REM Disconnect from the SA SDK Server.  
    REM *******************************************************************************************************
    Private Sub DisconnectButton_Click(sender As System.Object, e As System.EventArgs) Handles DisconnectButton.Click
        Me.Close()
    End Sub

    REM ///////////////////////////////////////////////////////////////////////////////////////////////////////
    REM *******************************************************************************************************
    REM Export all Point Groups of all Collecions to an ASCII file 
    REM *******************************************************************************************************
    REM ///////////////////////////////////////////////////////////////////////////////////////////////////////
    Private Sub RunTestButton_Click(sender As System.Object, e As System.EventArgs) Handles RunTestButton.Click
 
        MsgBox("SA SDK example 1: Export Points from all collections to ASCII File")

        Dim bSendStatus As Boolean = False
        Dim colTotal As Integer = 0
        Dim filename As String = "C:\Test\mypoints.txt"

        REM *****************************************************************************************
        REM Analysis Operations >> Get Number of Collections
        REM *****************************************************************************************
        NRKSdk.SetStep("Get Number of Collections")
        bSendStatus = NRKSdk.ExecuteStep()

        If (bSendStatus) Then
            Dim rCode As Long
            NRKSdk.GetMPStepResult(rCode)
            If (rCode <> MPStatus.DoneSuccess) Then
                Return
            End If
        End If

        REM Now get the answer
        NRKSdk.GetIntegerArg("Total Count", colTotal)
        REM ******************************************************************************************

        REM Loop through all collections and export point groups to a specified file (append)
        For colID As Integer = 0 To (colTotal - 1)

            REM get collection name
            Dim colName As String = ""

            REM ****************************************************************************************
            REM Analysis Operations >> Get i-th Collection Name
            REM ****************************************************************************************
            NRKSdk.SetStep("Get i-th Collection Name")
            NRKSdk.SetIntegerArg("Collection Index", colID)
            bSendStatus = NRKSdk.ExecuteStep()

            If (bSendStatus) Then
                Dim rCode As Long
                NRKSdk.GetMPStepResult(rCode)
                If (rCode <> MPStatus.DoneSuccess) Then
                    Return
                End If
            End If

            REM Now get the name
            NRKSdk.GetCollectionNameArg("Resultant Name", colName)
            REM MsgBox("Collection " & colName)
            REM ****************************************************************************************

            REM ****************************************************************************************
            REM Analysis Operations >> Make a Collection Object Name Ref List - By Type
            REM *****************************************************************************************
            NRKSdk.SetStep("Make a Collection Object Name Ref List - By Type")
            NRKSdk.SetStringArg("Collection", colName)
            NRKSdk.SetObjectTypeArg("Object Type", "Point Group")
            bSendStatus = NRKSdk.ExecuteStep()

            If (bSendStatus) Then
                Dim rCode As Long
                NRKSdk.GetMPStepResult(rCode)
                If (rCode <> MPStatus.DoneSuccess) Then
                    REM in case of failure skip to next collection
                    Continue For
                End If
            End If

            Dim grpList As Object
            If (NRKSdk.GetCollectionObjectNameRefListArg("Resultant Collection Object Name List", grpList)) Then
                Dim row
                Dim val As String
                For row = LBound(grpList, 1) To UBound(grpList, 1)
                    val = grpList(row)
                    Debug.WriteLine(val)
                Next
            Else
                REM in case of failure skip to next collection
                Continue For
            End If

            REM *******************************************************************************************************
            REM File Operations >> File Export >> Export ASCII Points
            REM *******************************************************************************************************
            NRKSdk.SetStep("Export ASCII Points")
            NRKSdk.SetFilePathArg("ASCII File Path", filename, False)

            REM set list of point groups of the specified collection
            Dim grpNameList = grpList

            Dim vGroupNameList As Object = New System.Runtime.InteropServices.VariantWrapper(grpNameList)
            NRKSdk.SetCollectionGroupNameRefListArg("Group Names to export", vGroupNameList)

            REM Available options: 
            REM "Space", "Comma", "Tab", 
            NRKSdk.SetExportDataDelimeterTypeArg("Data Delimiter", "Comma")

            REM Available options: 
            REM "Collection Group Target", "Group Target", "Target", "None",
            NRKSdk.SetExportTargetNameFormatArg("Target Name Format", "Target")

            REM Available options: 
            REM "Cartesian", "Cylindric", "Polar", 
            NRKSdk.SetCoordinateSystemTypeArg("Desired Coordinate System", "Cartesian")
            NRKSdk.SetBoolArg("Include Target Offsets?", True)
            NRKSdk.SetBoolArg("Include Target Comments?", True)
            NRKSdk.SetBoolArg("Include Timestamps?", True)
            NRKSdk.SetBoolArg("Include Tolerances?", True)
            NRKSdk.SetBoolArg("Include Coordinate Uncertainties?", True)
            NRKSdk.SetBoolArg("Include SA version and frame comments?", False)
            NRKSdk.SetBoolArg("Include Axis Comments?", True)
            NRKSdk.SetBoolArg("Include Export Format Info?", True)
            NRKSdk.SetBoolArg("Include Weights?", True)
            NRKSdk.SetBoolArg("Include Measurement Details?", True)
            NRKSdk.SetBoolArg("Maximum Precision (Scientific Notation)?", True)
            NRKSdk.SetIntegerArg("Decimal Precision", 6)
            NRKSdk.SetBoolArg("Append?", True)

            bSendStatus = NRKSdk.ExecuteStep()

            If (bSendStatus) Then
                Dim rCode As Long
                NRKSdk.GetMPStepResult(rCode)

                Dim msg As String
                msg = ""

                If (rCode = MPStatus.DoneSuccess) Then
                    msg = "All points of collection " + colName + " were added to " + filename
                    MsgBox(msg)
                Else
                    If (rCode = MPStatus.SdkError) Then
                        Dim msgArray As Object
                        REM Display text returned from execution attempt to user...
                        If (NRKSdk.GetMPStepMessages(msgArray)) Then
                            Dim idx
                            For idx = LBound(msgArray) To UBound(msgArray)
                                msg = msg + msgArray(idx) + Chr(10) + Chr(13)
                            Next
                            MsgBox(msg)
                        End If
                    Else
                        msg = "Failure with error code: " + rCode.ToString()
                        MsgBox(msg)
                    End If
                End If
            End If
            REM **********************************************************************************************
            REM clean up
            grpList = Nothing   REM prepare to next colection - de-refrence the list 

        Next colID
    End Sub

    REM ///////////////////////////////////////////////////////////////////////////////////////////////////////
    REM *******************************************************************************************************
    REM Export all Point Groups of all Collecions to an ASCII file  (function use)
    REM *******************************************************************************************************
    REM ///////////////////////////////////////////////////////////////////////////////////////////////////////
    Private Sub RunTestWithFunctionsCallButton_Click(sender As System.Object, e As System.EventArgs) Handles RunTestWithFunctionsCallButton.Click

        MsgBox("SA SDK example 2: Export Points from all collections to ASCII File (with functions)")

        Dim bSendStatus As Boolean = False
        Dim colTotal As Integer = 0
        Dim filename As String = "C:\Test\mypoints2.txt"


        REM **************************
        REM Get Number of Collections
        REM **************************
        colTotal = GetNumberOfCollections()
        MsgBox("Total Number of Collections is " & colTotal.ToString())

        REM Loop through all collections and export point groups to a specified file (append)
        For colID As Integer = 0 To (colTotal - 1)

            REM **********************
            REM Get Collection Name
            REM **********************
            Dim colName As String = GetCollectionNameByIndex(colID)

            REM ************************************
            REM Get Collection's point groups list
            REM ************************************
            Dim grpList As Object
            If (GetCollectionPointGroupsList(colName, grpList) <> True) Then
                REM in case of failure skip to next collection
                Continue For
            End If

            REM ***********************************
            REM Export points to a file
            REM ***********************************
            Dim grpNameList = grpList
   
            If (ExportASCIIPoints(filename, grpNameList)) Then
                MsgBox("All points of collection " + colName + " were added to " + filename)
            End If

            REM ***********************************
            REM clean up
            REM ***********************************
            grpList = Nothing   REM prepare to next colection - de-refrence the list 

        Next colID

    End Sub

    REM //////////////////////////////////////////////////////////////////////////////////////////////////////
    REM //////////////////////////////////////////////////////////////////////////////////////////////////////
    REM *******************************************************************************************************
    REM Public functions
    REM *******************************************************************************************************
    REM //////////////////////////////////////////////////////////////////////////////////////////////////////
    REM //////////////////////////////////////////////////////////////////////////////////////////////////////

    REM *****************************************************************************************
    REM GetNumberOfCollections()
    REM *****************************************************************************************
    Public Function GetNumberOfCollections() As Integer
        Dim bSendStatus As Boolean
        Dim colTotal As Integer
        colTotal = 0

        REM ***************************************************************************************
        REM Analysis Operations >> Get Number of Collections
        REM ***************************************************************************************
        NRKSdk.SetStep("Get Number of Collections")
        bSendStatus = NRKSdk.ExecuteStep()

        If (bSendStatus) Then
            Dim rCode As Long
            NRKSdk.GetMPStepResult(rCode)
            If (rCode <> MPStatus.DoneSuccess) Then
                Return colTotal
            End If
        End If

        REM Now get the answer
        NRKSdk.GetIntegerArg("Total Count", colTotal)
        Return colTotal
    End Function

    REM *****************************************************************************************
    REM GetCollectionNameByIndex()
    REM *****************************************************************************************
    Public Function GetCollectionNameByIndex(idx As Integer) As String
        Dim bSendStatus As Boolean
        Dim colName As String = ""

        REM ****************************************************
        REM Analysis Operations >> Get i-th Collection Name
        REM ****************************************************
        NRKSdk.SetStep("Get i-th Collection Name")
        NRKSdk.SetIntegerArg("Collection Index", idx)
        bSendStatus = NRKSdk.ExecuteStep()

        If (bSendStatus) Then
            Dim rCode As Long
            NRKSdk.GetMPStepResult(rCode)
            If (rCode <> MPStatus.DoneSuccess) Then
                Return colName
            End If
        End If

        REM Now get the name
        NRKSdk.GetCollectionNameArg("Resultant Name", colName)
        Return colName
    End Function

    REM *****************************************************************************************
    REM GetCollectionPointGroupsList()
    REM *****************************************************************************************
    Public Function GetCollectionPointGroupsList(colName As String, ByRef ptsGrps As Object) As Boolean
        Dim bSendStatus As Boolean
  
        REM **************************************************************************************
        REM Analysis Operations >> Make a Collection Object Name Ref List - By Type
        REM **************************************************************************************
        NRKSdk.SetStep("Make a Collection Object Name Ref List - By Type")
        NRKSdk.SetStringArg("Collection", colName)
        NRKSdk.SetObjectTypeArg("Object Type", "Point Group")
        bSendStatus = NRKSdk.ExecuteStep()

        If (bSendStatus) Then
            Dim rCode As Long
            NRKSdk.GetMPStepResult(rCode)
            If (rCode <> MPStatus.DoneSuccess) Then
                REM in case of failure skip to next collection
                Return False
            End If
        End If

        Dim grpNameList As Object
        If (NRKSdk.GetCollectionObjectNameRefListArg("Resultant Collection Object Name List", grpNameList)) Then
            Dim row
            Dim val As String
            For row = LBound(grpNameList, 1) To UBound(grpNameList, 1)
                val = grpNameList(row)
                Debug.WriteLine(val)
            Next
        Else
            REM in case of failure skip to next collection
            Return False
        End If

        ptsGrps = grpNameList
        Return True
    End Function

    REM *****************************************************************************************
    REM ExportASCIIPoints()
    REM *****************************************************************************************
    Public Function ExportASCIIPoints(filename As String, ByRef ptsGrps As Object) As Boolean
        Dim bSendStatus As Boolean

        REM *******************************************************************************************************
        REM File Operations >> File Export >> Export ASCII Points
        REM *******************************************************************************************************
        NRKSdk.SetStep("Export ASCII Points")
        NRKSdk.SetFilePathArg("ASCII File Path", filename, False)

        Dim grpNameList = ptsGrps
        Dim vGroupNameList As Object = New System.Runtime.InteropServices.VariantWrapper(grpNameList)
        NRKSdk.SetCollectionGroupNameRefListArg("Group Names to export", vGroupNameList)

        NRKSdk.SetExportDataDelimeterTypeArg("Data Delimiter", "Comma")
        NRKSdk.SetExportTargetNameFormatArg("Target Name Format", "Target")
        NRKSdk.SetCoordinateSystemTypeArg("Desired Coordinate System", "Cartesian")
        NRKSdk.SetBoolArg("Include Target Offsets?", True)
        NRKSdk.SetBoolArg("Include Target Comments?", True)
        NRKSdk.SetBoolArg("Include Timestamps?", True)
        NRKSdk.SetBoolArg("Include Tolerances?", True)
        NRKSdk.SetBoolArg("Include Coordinate Uncertainties?", True)
        NRKSdk.SetBoolArg("Include SA version and frame comments?", False)
        NRKSdk.SetBoolArg("Include Axis Comments?", True)
        NRKSdk.SetBoolArg("Include Export Format Info?", True)
        NRKSdk.SetBoolArg("Include Weights?", True)
        NRKSdk.SetBoolArg("Include Measurement Details?", True)
        NRKSdk.SetBoolArg("Maximum Precision (Scientific Notation)?", True)
        NRKSdk.SetIntegerArg("Decimal Precision", 6)
        NRKSdk.SetBoolArg("Append?", True)

        bSendStatus = NRKSdk.ExecuteStep()

        If (bSendStatus) Then
            Dim rCode As Long
            NRKSdk.GetMPStepResult(rCode)

            If (rCode = MPStatus.DoneSuccess) Then
                Return True
            Else
                Dim msg As String = ""
                If (rCode = MPStatus.SdkError) Then
                    Dim msgArray As Object

                    REM Display text returned from execution attempt to user...
                    If (NRKSdk.GetMPStepMessages(msgArray)) Then
                        Dim idx
                        For idx = LBound(msgArray) To UBound(msgArray)
                            msg = msg + msgArray(idx) + Chr(10) + Chr(13)
                        Next
                        MsgBox(msg)
                    End If
                Else
                    msg = "Failure with error code: " + rCode.ToString()
                    MsgBox(msg)
                End If
            End If
        End If

        Return False
    End Function

 
End Class
